
#
# 2014.10.20 jdwang@asia.edu.tw
#
# usage : perl PubMed_InfoExtration_14_ArticlesStatistics_TFIDF_Wordle_7.pl  "../SrcData_Topic_WilsonDisease_Parsed" "../SrcData_Topic_WilsonDisease_Statistics" "../stop-words/stop-words_english_3_en.txt" "../stop-words/jdwang_stop-words.txt" 100


use strict;
use Data::Dumper;
use jdwangPubMedPackage3;
use FileHandle;

my $SrcDataDir = $ARGV[0];
my $OutPutDir = $ARGV[1];
my $Stop_WordLocation = $ARGV[2];
my $jdwang_Stop_WordLocation = $ARGV[3];
my $Limit_NumOfWord = $ARGV[4];


my %StopWords = ();
#
# the general case of stop-words in English
#
my $fh_stopword = new FileHandle($Stop_WordLocation,"r") or die "can open $Stop_WordLocation";
while (my $OneStopWord  = <$fh_stopword> ) {	
	chomp($OneStopWord);
	#print Dumper($OneStopWord);
	$StopWords{$OneStopWord}=0;
}
$fh_stopword->close();
#
# the special case of stop-words (jdwang 2014.11.3)
#
my $fh_stopword1 = new FileHandle($jdwang_Stop_WordLocation,"r") or die "can open $jdwang_Stop_WordLocation";
while (my $OneStopWord  = <$fh_stopword1> ) {
	chomp($OneStopWord);
	#print Dumper($OneStopWord);
	$StopWords{$OneStopWord}=0;
}
$fh_stopword1->close();
#print Dumper(%StopWords);exit;

my %ArticleWordsStatistics = ();
#
# the number of articles
#
my $ArticleCnt = 0;

my @SegFileAry = ();
my $SeqFilesCnt = jdwangPubMedPackage3::DirFilenames($SrcDataDir,\@SegFileAry);
#print Dumper(@SegFileAry);exit;	

foreach my $OneFile (@SegFileAry) {
		my $InputFileLocation  = $SrcDataDir."/".$OneFile;		
		#print "$InputFileLocation\n";		
			
		print "begin $OneFile ".scalar localtime(time())."\n";#exit;	
		
		my $fh_input = new FileHandle($InputFileLocation,"r");	
			
		my %OneArticleWordsStatistics = ();
		
		while (my $Line = <$fh_input>) {
			#25276143##2014-12-3##Fulminant Wilson's Disease Managed with Plasmapheresis as a Bridge to Liver Transplant.##New-onset jaundice can be a manifestation of multiple pathologic processes including hemolysis, parenchymal liver disease, and cholestasis; the differential diagnosis is broad and requires a systematic approach. We report a case of a patient who presented with jaundice after starting minocycline for the treatment of acne vulgaris and rapidly developed fulminant liver failure found to be due to Wilson's disease. She also manifested severe Coomb's negative hemolytic anemia and renal failure secondary to hepatorenal syndrome. As a bridge to liver transplant, she was successfully treated with plasmapheresis to decrease serum copper in addition to hemodialysis for acidosis and hyperkalemia. She was able to receive a liver and made a full recovery. The case highlights the use of plasmapheresis as an adjunctive treatment modality in cases of fulminant liver failure due to Wilson's disease.
			#print $Line; exit;
			$ArticleCnt++;
			
			chomp($Line);				
			
			my @Items = split (/\#\#/,$Line );
			#print Dumper(@Items);exit;
			my $PMID = $Items[0];
			print "($ArticleCnt)PMID=>$PMID\n";
			my $Date = $Items[1];	
			my $Title = $Items[2];
			my $Abstract = $Items[3];
			
			my $Content = $Title." ".$Abstract;
			
			$Content  =~ s/\, |\. |\; |\.\x0D\x0A|\? |\: |\./\#/g;
	    
	    	#print Dumper($Content);exit;
	    
	    	my $SentenceSeparators = '#';
			
			my @Sentences = split ($SentenceSeparators,$Content);
									
			#print Dumper(@Sentence);exit;
			
			foreach my $OneSentence (@Sentences) {
				#print Dumper($OneSentence);
				if ($OneSentence ne '') {
					#print Dumper($OneSentence);exit;
					my @Words = split ( / /,$OneSentence);
				    #print Dumper(@Words);exit;
				    
				    # 
				    # record the term frequency (tf) of each "word"
				    #
				    
				    foreach my $OneWord (@Words) {
					    if (( $OneWord ne '')&&(! exists($StopWords{$OneWord}))) {
					   		$OneArticleWordsStatistics{$OneWord}++; 
				   		} # end of if (( $OneWord ne '')&&(! exists($StopWords{$OneWord}))) 
				    	#print Dumper(%ArticleWordsStatistics);exit; 
				    } # end of foreach my $OneWord (@Words)
				    #print Dumper(%OneArticleWordsStatistics);exit;
				} else {
					# the sentence is empty
					#print Dumper($OneSentence);
				}# end of if ($OneSentence ne '')
			} # end of foreach $OneSentence (@Sentence)
			#print Dumper(%OneArticleWordsStatistics);exit;
			foreach my $OneWord (keys %OneArticleWordsStatistics) {
				$ArticleWordsStatistics{$OneWord}{'tf'} += $OneArticleWordsStatistics{$OneWord};
				$ArticleWordsStatistics{$OneWord}{'df'}++;
			} # end of foreach my $OneWord (keys %OneArticleWordsStatistics)
			#print Dumper(%ArticleWordsStatistics);exit;
		} # end of while (my $Line = <$fh_input>)	
		$fh_input->close();	
			
		#print Dumper(%ArticleWordsStatistics);exit;	
		print "end $OneFile ".scalar localtime(time())."\n";
		#print Dumper(%ArticleStatistics);exit;		
} # end of foreach my $OneFile (@SegFileAry)
#print Dumper(%ArticleWordsStatistics);exit;




#
# compute the TF*IDF of $OneWord
#
my %WordWeight = ();
foreach my $OneWord (keys %ArticleWordsStatistics) {	
	$WordWeight{$OneWord} = $ArticleWordsStatistics{$OneWord}{'tf'} * log ($ArticleCnt/$ArticleWordsStatistics{$OneWord}{'df'});
} # end of foreach my $OneWord (keys %ArticleWordsStatistics)
#print Dumper(%WordWeight);exi


#
# output the term frequency (tf) of each word to file
# 
mkdir $OutPutDir;
my $OutputFileLocation = $OutPutDir."/"."ArticlesWordStatistics_14_7_OwnStopwords_Top".$Limit_NumOfWord.".txt";
my $fh_output = new FileHandle($OutputFileLocation, "w") or die "can't open $OutputFileLocation";

print $fh_output "Pattern:TFIDF\n";

#
# output the statistics of words ordered by the TF*IDF@weighting.
#
my $cnt = 0;
foreach my $OneWord (sort {$WordWeight{$b} <=> $WordWeight{$a}} keys %WordWeight){	
	$cnt++;
	if ($cnt <= $Limit_NumOfWord) {
		#print "$cnt\t$OneWord\t$ArticleWordsStatistics{$OneWord}{'tf'}\t$ArticleWordsStatistics{$OneWord}{'df'}\t$WordWeight{$OneWord}\n";
		#print $fh_output "$cnt\t$OneWord\t$ArticleWordsStatistics{$OneWord}{'tf'}\t$ArticleWordsStatistics{$OneWord}{'df'}\t$WordWeight{$OneWord}\n";
		print  "$OneWord:$WordWeight{$OneWord}\n";
		print $fh_output "$OneWord:$WordWeight{$OneWord}\n";
	} # end of if ($cnt <= $Limit_NumOfWord)
} # end of foreach my $OneWord (sort {$WordWeight{$b} <=> $WordWeight{$a}} keys %WordWeight)
$fh_output->close;















